# pylint: disable=missing-module-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=broad-exception-caught
# pylint: disable=line-too-long
# pylint: disable=unused-variable
# pylint: disable=broad-exception-caught
# pylint: disable=no-self-argument  # Added for Pydantic validators
from datetime import datetime
from typing import Set, Optional, ClassVar
from pydantic import BaseModel, field_validator, Field, ConfigDict
import uuid
import dateutil.parser
from ipulse_shared_base_ftredge import (
    OrganizationRelation,
    OrganizationIndustry
)

class Organisation(BaseModel):
    """
    Organisation model representing business entities in the system.
    Supports both retail and non-retail customer types with different validation rules.
    """
    model_config = ConfigDict(frozen=True, extra="forbid")
    
    # Class constants
    VERSION: ClassVar[float] = 1.0
    MODULE: ClassVar[str] = "core"
    CLASS_REF: ClassVar[str] = "orgn"

    # Required fields
    puid: str = Field(
        default_factory=lambda: f"{datetime.utcnow().strftime('%Y%m%d%H%M%S')}_{uuid.uuid4().hex[:8]}_{Organisation.MODULE}{Organisation.CLASS_REF}".lower(),
        description="Unique identifier for the organisation"
    )
    name: str = Field(..., min_length=1, max_length=100)
    relations: Set[OrganizationRelation] = Field(..., description="Organisation relations/types")

    # Timestamps
    creat_date: datetime = Field(default_factory=datetime.utcnow)
    updt_date: datetime = Field(default_factory=datetime.utcnow)
    
    # Optional fields
    creat_by_user: Optional[str] = Field(None, max_length=100)
    updt_by_user: Optional[str] = Field(None, max_length=100)
    description: Optional[str] = Field(None, max_length=1000)
    industries: Optional[Set[OrganizationIndustry]] = None
    website: Optional[str] = Field(None, max_length=200)
    org_admin_user_uids: Optional[Set[str]] = None

    @field_validator('relations')
    @classmethod
    def validate_relations(cls, v: Set[OrganizationRelation]) -> Set[OrganizationRelation]:
        return v

    @field_validator('industries')
    @classmethod
    def validate_industries(cls, v: Optional[Set[OrganizationIndustry]], info) -> Optional[Set[OrganizationIndustry]]:
        values = info.data
        is_retail = values.get('relations') == {OrganizationRelation.RETAIL_CUSTOMER}
        if is_retail and v is not None:
            raise ValueError("Industries should not be set for retail customers")
        elif not is_retail and v is None:
            raise ValueError("Industries required for non-retail customers")
        return v

    @field_validator('website', 'description')
    @classmethod
    def validate_retail_fields(cls, v: Optional[str], info) -> Optional[str]:
        values = info.data
        field = info.field_name
        is_retail = values.get('relations') == {OrganizationRelation.RETAIL_CUSTOMER}
        if is_retail and v is not None:
            raise ValueError(f"{field} should not be set for retail customers")
        elif not is_retail and not v:
            raise ValueError(f"{field} required for non-retail customers")
        return v

    @field_validator('creat_date', 'updt_date', mode='before')
    @classmethod
    def parse_datetime(cls, v: any) -> datetime:
        if isinstance(v, datetime):
            return v
        try:
            return dateutil.parser.isoparse(v)
        except (TypeError, ValueError) as e:
            raise ValueError(f"Invalid datetime format: {e}")